@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.123 2006/08/02 13:44:26 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.125 2006/08/22 06:47:25 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -62,7 +62,9 @@ static ResDecl
    res_move_background	   ("cmd:move-background","yes", ResMgr::BoolValidate,ResMgr::NoClosure),
    res_set_term_status     ("cmd:set-term-status","no", ResMgr::BoolValidate,0),
    res_term_status         ("cmd:term-status",  "", 0, 0),
-   res_trace		   ("cmd:trace",  "no",	ResMgr::BoolValidate,ResMgr::NoClosure);
+   res_trace		   ("cmd:trace",  "no",	ResMgr::BoolValidate,ResMgr::NoClosure),
+   res_parallel		   ("cmd:parallel",	   "1",  ResMgr::UNumberValidate,0),
+   res_queue_parallel	   ("cmd:queue-parallel",  "1",	 ResMgr::UNumberValidate,0);
 
 CmdExec	 *CmdExec::cwd_owner;
 CmdExec	 *CmdExec::chain;
@@ -329,7 +331,7 @@ void CmdExec::RemoveFeeder()
       queue_feeder=0;
    delete feeder;
    feeder=tmp;
-
+   Reconfig(0);
 }
 
 void CmdExec::ReuseSavedSession()
@@ -590,7 +592,8 @@ int CmdExec::Do()
       }
       if(status_line && status_line->CanShowNow())
 	 ShowRunStatus(status_line);   // this is only for top level CmdExec.
-      return m;
+      if(m != STALL || interactive || !feeder || waiting_num >= max_waiting)
+	 return m;
    }
 
    if(!interactive)
@@ -644,6 +647,8 @@ try_get_cmd:
 	       FeedCmd("exit;");
 	       return MOVED;
 	    }
+	    if(waiting_num > 0)
+	      return m;
 	    RemoveFeeder();
 	    m=MOVED;
 	    goto try_get_cmd;
@@ -824,6 +829,7 @@ CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f?f:new DummyProt
    redirections=0;
 
    queue_feeder=0;
+   max_waiting=1;
 
    saved_session=0;
 
@@ -955,6 +961,8 @@ void CmdExec::Reconfig(const char *name)
    // only allow explicit setting of cmd:interactive to change interactiveness.
    if(top_level && name && !strcmp(name,"cmd:interactive"))
       SetInteractive(res_interactive.QueryBool(0));
+   ResType *r=queue_feeder?&res_queue_parallel:&res_parallel;
+   max_waiting=r->Query(c);
 }
 
 void CmdExec::pre_stdout()
@@ -1340,6 +1348,7 @@ CmdExec  *CmdExec::GetQueue(bool create)
    queue->cmdline=xasprintf("queue (%s%s%s)",url,slot?"; ":"",slot?slot:"");
    queue->queue_feeder=new QueueFeeder(session->GetCwd(), cwd->GetName());
    queue->SetCmdFeeder(queue->queue_feeder);
+   queue->Reconfig(0);
 
    return queue;
 }
