@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.91 2005/12/31 07:01:36 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.96 2006/07/28 06:13:21 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -741,8 +741,12 @@ static int parse_html(const char *buf,int len,bool eof,Buffer *list,
    if(tag_scan->tag==0)
       return tag_len;	// not interesting
 
-   bool hftp=(prefix && (!xstrcmp(prefix->proto,"hftp")
-		      || !xstrcmp(prefix->proto,"ftp")));
+   const char *prefix_proto=0;
+   if(prefix)
+      prefix_proto=prefix->proto;
+   if(!xstrcmp(prefix_proto,"hftp"))
+      prefix_proto++;
+   bool hftp=!xstrcmp(prefix_proto,"ftp");
 
    // ok, found the target.
 
@@ -770,7 +774,7 @@ static int parse_html(const char *buf,int len,bool eof,Buffer *list,
       {
 	 xfree(*base_href);
 	 *base_href=xstrdup(link_target,+2);
-	 Log::global->Format(10,"Found base href=%s\n",*base_href);
+	 Log::global->Format(10,"Using base href=%s\n",*base_href);
       }
       return tag_len;
    }
@@ -832,7 +836,7 @@ parse_url_again:
       if(!prefix)
 	 return tag_len;	// no way
 
-      if(xstrcmp(link_url.proto,prefix->proto+hftp)
+      if(xstrcmp(link_url.proto,prefix_proto)
       || xstrcmp(link_url.host,prefix->host)
       || xstrcmp(link_url.user,prefix->user)
       || xstrcmp(link_url.port,prefix->port))
@@ -1072,7 +1076,7 @@ parse_url_again:
 	 struct tm tm;
 	 memset(&tm,0,sizeof(tm));
 	 tm.tm_year=info.year-1900;
-	 tm.tm_mon=info.month-1;
+	 tm.tm_mon=info.month;
 	 tm.tm_mday=info.day;
 	 tm.tm_hour=12;
 	 info.date_prec=43200;
@@ -1264,7 +1268,7 @@ int HttpDirList::Do()
       const char *cache_buffer=0;
       int cache_buffer_size=0;
       int err;
-      if(use_cache && LsCache::Find(session,curr,mode,&err,
+      if(use_cache && FileAccess::cache->Find(session,curr,mode,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
 	 if(err)
@@ -1290,8 +1294,8 @@ int HttpDirList::Do()
 	 session->Open(curr,mode);
 	 session->UseCache(use_cache);
 	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
+	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
+	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
 
@@ -1300,7 +1304,7 @@ int HttpDirList::Do()
    ubuf->Get(&b,&len);
    if(b==0) // eof
    {
-      LsCache::Add(session,curr,mode,FA::OK,ubuf);
+      FileAccess::cache->Add(session,curr,mode,FA::OK,ubuf);
       Delete(ubuf);
       ubuf=0;
       return MOVED;
@@ -1328,7 +1332,7 @@ reparse:
 
    if(ubuf->Error())
    {
-      LsCache::Add(session,curr,mode,session->GetErrorCode(),ubuf);
+      FileAccess::cache->Add(session,curr,mode,session->GetErrorCode(),ubuf);
       if(mode==FA::MP_LIST)
       {
 	 mode=FA::LONG_LIST;
@@ -1429,7 +1433,7 @@ FileSet *HttpListInfo::Parse(const char *b,int len)
    return session->ParseLongList(b,len);
 }
 
-FileSet *Http::ParseLongList(const char *b,int len,int *err)
+FileSet *Http::ParseLongList(const char *b,int len,int *err) const
 {
    if(err)
       *err=0;
