@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.h,v 1.15.2.1 2002/01/15 16:20:22 lav Exp $ */
+/* $Id: Job.h,v 1.26 2006/06/13 14:35:38 lav Exp $ */
 
 #ifndef JOB_H
 #define JOB_H
@@ -61,14 +61,21 @@ public:
    void SetParentFg(Job *j, bool f=true)
       {
 	 SetParent(j);
-	 if(j->fg && f)
+	 if(f && j->fg)
 	    Fg();
+//	 else if(f && !j->fg)
+//	    Bg();
       }
 
    void	 AllocJobno();
 
-   virtual void	  PrintStatus(int) {};
+   virtual void	  PrintStatus(int,const char *prefix="\t") {};
    virtual void	  ShowRunStatus(StatusLine *);
+   void ClearStatus()
+      {
+	 const char *empty="";
+	 eprintf(empty);
+      }
    virtual void	  SayFinal() {}; // final phrase of fg job
    virtual int	  Done()=0;
    virtual int	  ExitCode() { return 0; }
@@ -87,6 +94,7 @@ public:
    Job *FindAnyChild();
    bool WaitsFor(Job *);
    static Job *FindWhoWaitsFor(Job *);
+   bool CheckForWaitLoop(Job *parent);
    int NumAwaitedJobs() { return waiting_num; }
    Job *FindDoneAwaitedJob();
    void WaitForAllChildren();
@@ -99,8 +107,10 @@ public:
       return j && !j->Done();
    }
    void Kill(int n);
+   static void Kill(Job*);
    void SendSig(int n,int sig);
-   void KillAll();
+   static void KillAll();
+   static void Cleanup();
 
    void vfprintf(FILE *file,const char *fmt,va_list v);
    // CmdExec redefines this, and traps all messages of its children.
@@ -116,21 +126,26 @@ public:
    Job();
 
    virtual const char *GetConnectURL() { return 0; }
-   virtual void lftpMovesToBackground() {}
+   virtual void lftpMovesToBackground() { Resume(); }
    static  void lftpMovesToBackground_ToAll();
+
+   virtual off_t GetBytesCount() { return 0; }
+   virtual double GetTimeSpent() { return 0; }
+
+   void WaitDone();
 };
 
 class SessionJob : public Job
 {
 protected:
    static void Reuse(FileAccess *s) { SessionPool::Reuse(s); }
-   FileAccess *Clone();
+   FileAccess *Clone() const;
 
    SessionJob(FileAccess *);
    ~SessionJob();
 
 public:
-   void PrintStatus(int v);
+   void PrintStatus(int v,const char *);
    FileAccess *session;
    const char *GetConnectURL()
       {
