@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.130 2005/12/15 08:49:47 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.139 2006/08/29 07:25:26 lav Exp $ */
 
 #include <config.h>
 
@@ -35,7 +35,6 @@
 #include "ChmodJob.h"
 #include "misc.h"
 #include "plural.h"
-#include "getopt.h"
 #include "FindJob.h"
 #include "url.h"
 #include "CopyJob.h"
@@ -152,7 +151,7 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
       if(target_list_info && (!source_list_info || now%4>=2))
       {
 	 const char *status=target_list_info->Status();
-	 int status_w=mbswidth(status, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+	 int status_w=mbswidth(status, 0);
 	 int dw=w-status_w;
 	 if(dw<20)
 	    dw=20;
@@ -164,7 +163,7 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
       else if(source_list_info)
       {
 	 const char *status=source_list_info->Status();
-	 int status_w=mbswidth(status, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+	 int status_w=mbswidth(status, 0);
 	 int dw=w-status_w;
 	 if(dw<20)
 	    dw=20;
@@ -298,7 +297,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       c->RemoveTargetFirst();
 	    CopyJob *cp=
 	       new CopyJob(c,file->name,"mirror");
-	    if(file->defined&file->DATE)
+	    if((file->defined&file->DATE) && file->date_prec<=1)
 	       cp->SetDate(file->date);
 	    if(file->defined&file->SIZE)
 	       cp->SetSize(file->size);
@@ -312,7 +311,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    ArgV *get_args=new ArgV("mirror");
 	    get_args->Append(file->name);
 	    get_args->Append(dir_file(target_dir,file->name));
-	    pgetJob *cp=new pgetJob(source_session->Clone(),get_args);
+	    pgetJob *cp=new pgetJob(source_session->Clone(),get_args,cont_this);
 	    cp->SetMaxConn(pget_n);
 	    if(remove_source_files)
 	       cp->RemoveSourceLater();
@@ -385,6 +384,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 mj->pget_n=pget_n;
 	 mj->pget_minchunk=pget_minchunk;
 	 mj->remove_source_files=remove_source_files;
+	 mj->skip_noaccess=skip_noaccess;
 	 mj->create_target_dir=create_target_dir;
 	 mj->no_target_dir=no_target_dir;
 
@@ -496,6 +496,9 @@ void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
    if(flags&NO_RECURSION)
       to_transfer->SubtractDirs();
 
+   if(skip_noaccess)
+      to_transfer->ExcludeUnaccessible();
+
    new_files_set=new FileSet(to_transfer);
    new_files_set->SubtractAny(dest);
    old_files_set=new FileSet(dest);
@@ -1057,6 +1060,7 @@ MirrorJob::MirrorJob(MirrorJob *parent,
 
    use_cache=false;
    remove_source_files=false;
+   skip_noaccess=false;
 
    parallel=1;
    pget_n=1;
@@ -1262,11 +1266,32 @@ CMD(mirror)
 {
 #define args (parent->args)
 #define eprintf parent->eprintf
-   static struct option mirror_opts[]=
+   enum {
+      OPT_ALLOW_CHOWN,
+      OPT_DELETE_FIRST,
+      OPT_IGNORE_SIZE,
+      OPT_IGNORE_TIME,
+      OPT_LOOP,
+      OPT_MAX_ERRORS,
+      OPT_NO_DEREFERENCE,
+      OPT_NO_SYMLINKS,
+      OPT_NO_UMASK,
+      OPT_OLDER_THAN,
+      OPT_ONLY_MISSING,
+      OPT_PERMS,
+      OPT_REMOVE_SOURCE_FILES,
+      OPT_SCRIPT,
+      OPT_SCRIPT_ONLY,
+      OPT_SIZE_RANGE,
+      OPT_USE_CACHE,
+      OPT_USE_PGET_N,
+      OPT_SKIP_NOACCESS,
+   };
+   static const struct option mirror_opts[]=
    {
       {"delete",no_argument,0,'e'},
       {"allow-suid",no_argument,0,'s'},
-      {"allow-chown",no_argument,0,256+'a'},
+      {"allow-chown",no_argument,0,OPT_ALLOW_CHOWN},
       {"include",required_argument,0,'i'},
       {"exclude",required_argument,0,'x'},
       {"include-glob",required_argument,0,'I'},
@@ -1274,29 +1299,32 @@ CMD(mirror)
       {"only-newer",no_argument,0,'n'},
       {"no-recursion",no_argument,0,'r'},
       {"no-perms",no_argument,0,'p'},
-      {"no-umask",no_argument,0,256+'u'},
+      {"perms",no_argument,0,OPT_PERMS},
+      {"no-umask",no_argument,0,OPT_NO_UMASK},
       {"continue",no_argument,0,'c'},
       {"reverse",no_argument,0,'R'},
       {"verbose",optional_argument,0,'v'},
       {"newer-than",required_argument,0,'N'},
-      {"older-than",required_argument,0,256+'O'},
-      {"size-range",required_argument,0,512+'S'},
+      {"older-than",required_argument,0,OPT_OLDER_THAN},
+      {"size-range",required_argument,0,OPT_SIZE_RANGE},
       {"dereference",no_argument,0,'L'},
-      {"use-cache",no_argument,0,256+'C'},
-      {"Remove-source-files",no_argument,0,256+'R'},
+      {"no-dereference",no_argument,0,OPT_NO_DEREFERENCE},
+      {"use-cache",no_argument,0,OPT_USE_CACHE},
+      {"Remove-source-files",no_argument,0,OPT_REMOVE_SOURCE_FILES},
       {"parallel",optional_argument,0,'P'},
-      {"ignore-time",no_argument,0,256+'i'},
-      {"ignore-size",no_argument,0,257+'i'},
-      {"only-missing",no_argument,0,256+'m'},
-      {"log",required_argument,0,256+'s'},
-      {"script",    required_argument,0,256+'S'},
-      {"just-print",optional_argument,0,256+'S'},
-      {"dry-run",   optional_argument,0,256+'S'},
-      {"delete-first",no_argument,0,256+'e'},
-      {"use-pget-n",optional_argument,0,256+'p'},
-      {"no-symlinks",no_argument,0,256+'y'},
-      {"loop",no_argument,0,256+'l'},
-      {"max-errors",required_argument,0,256+'E'},
+      {"ignore-time",no_argument,0,OPT_IGNORE_TIME},
+      {"ignore-size",no_argument,0,OPT_IGNORE_SIZE},
+      {"only-missing",no_argument,0,OPT_ONLY_MISSING},
+      {"log",required_argument,0,OPT_SCRIPT},
+      {"script",    required_argument,0,OPT_SCRIPT_ONLY},
+      {"just-print",optional_argument,0,OPT_SCRIPT_ONLY},
+      {"dry-run",   optional_argument,0,OPT_SCRIPT_ONLY},
+      {"delete-first",no_argument,0,OPT_DELETE_FIRST},
+      {"use-pget-n",optional_argument,0,OPT_USE_PGET_N},
+      {"no-symlinks",no_argument,0,OPT_NO_SYMLINKS},
+      {"loop",no_argument,0,OPT_LOOP},
+      {"max-errors",required_argument,0,OPT_MAX_ERRORS},
+      {"skip-noaccess",no_argument,0,OPT_SKIP_NOACCESS},
       {0}
    };
 
@@ -1314,6 +1342,7 @@ CMD(mirror)
    const char *older_than=0;
    Range *size_range=0;
    bool  remove_source_files=false;
+   bool	 skip_noaccess=ResMgr::QueryBool("mirror:skip-noaccess",0);
    int	 parallel=ResMgr::Query("mirror:parallel-transfer-count",0);
    int	 use_pget=ResMgr::Query("mirror:use-pget-n",0);
    bool	 reverse=false;
@@ -1322,9 +1351,12 @@ CMD(mirror)
 
    PatternSet *exclude=0;
    const char *default_exclude=ResMgr::Query("mirror:exclude-regex",0);
+   const char *default_include=ResMgr::Query("mirror:include-regex",0);
 
    if(!ResMgr::QueryBool("mirror:set-permissions",0))
       flags|=MirrorJob::NO_PERMS;
+   if(ResMgr::QueryBool("mirror:dereference",0))
+      flags|=MirrorJob::RETR_SYMLINKS;
 
    args->rewind();
    while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LPa",mirror_opts,0))!=EOF)
@@ -1337,7 +1369,7 @@ CMD(mirror)
       case('s'):
 	 flags|=MirrorJob::ALLOW_SUID;
 	 break;
-      case(256+'a'):
+      case(OPT_ALLOW_CHOWN):
 	 flags|=MirrorJob::ALLOW_CHOWN;
 	 break;
       case('a'):
@@ -1352,6 +1384,9 @@ CMD(mirror)
       case('p'):
 	 flags|=MirrorJob::NO_PERMS;
 	 break;
+      case(OPT_PERMS):
+	 flags&=~MirrorJob::NO_PERMS;
+	 break;
       case('c'):
 	 flags|=MirrorJob::CONTINUE;
 	 break;
@@ -1387,7 +1422,11 @@ CMD(mirror)
 	     * explicit pattern is for exclusion - otherwise all files are
 	     * excluded by default and no default exclusion is needed. */
 	    if(type==PatternSet::EXCLUDE && default_exclude && *default_exclude)
+	    {
 	       exclude->Add(type,new PatternSet::Regex(default_exclude));
+	       if(default_include && *default_include)
+		  exclude->Add(PatternSet::INCLUDE,new PatternSet::Regex(default_include));
+	    }
 	    default_exclude=0;
 	    /* Users usually don't want to exclude all directories */
 	    if(type==PatternSet::INCLUDE)
@@ -1402,6 +1441,9 @@ CMD(mirror)
       case('L'):
 	 flags|=MirrorJob::RETR_SYMLINKS;
 	 break;
+      case(OPT_NO_DEREFERENCE):
+	 flags&=~MirrorJob::RETR_SYMLINKS;
+	 break;
       case('v'):
 	 if(optarg)
 	    verbose=atoi(optarg);
@@ -1413,10 +1455,10 @@ CMD(mirror)
       case('N'):
 	 newer_than=optarg;
 	 break;
-      case(256+'O'):
+      case(OPT_OLDER_THAN):
 	 older_than=optarg;
 	 break;
-      case(512+'S'):
+      case(OPT_SIZE_RANGE):
 	 size_range=new Range(optarg);
 	 if(size_range->Error())
 	 {
@@ -1425,22 +1467,22 @@ CMD(mirror)
 	    goto no_job;
 	 }
 	 break;
-      case(256+'u'):
+      case(OPT_NO_UMASK):
 	 flags|=MirrorJob::NO_UMASK;
 	 break;
-      case(256+'C'):
+      case(OPT_USE_CACHE):
 	 use_cache=true;
 	 break;
-      case(256+'R'):
+      case(OPT_REMOVE_SOURCE_FILES):
 	 remove_source_files=true;
 	 break;
-      case(256+'i'):
+      case(OPT_IGNORE_TIME):
 	 flags|=MirrorJob::IGNORE_TIME;
 	 break;
-      case(257+'i'):
+      case(OPT_IGNORE_SIZE):
 	 flags|=MirrorJob::IGNORE_SIZE;
 	 break;
-      case(256+'m'):
+      case(OPT_ONLY_MISSING):
 	 flags|=MirrorJob::IGNORE_TIME|MirrorJob::IGNORE_SIZE;
 	 break;
       case('P'):
@@ -1449,31 +1491,34 @@ CMD(mirror)
 	 else
 	    parallel=3;
 	 break;
-      case(256+'p'):
+      case(OPT_USE_PGET_N):
 	 if(optarg)
 	    use_pget=atoi(optarg);
 	 else
 	    use_pget=3;
 	 break;
-      case(256+'S'):
+      case(OPT_SCRIPT_ONLY):
 	 script_only=true;
-      case(256+'s'):
+      case(OPT_SCRIPT):
 	 script_file=optarg;
 	 if(script_file==0)
 	    script_file="-";
 	 break;
-      case(256+'e'):
+      case(OPT_DELETE_FIRST):
 	 flags|=MirrorJob::REMOVE_FIRST|MirrorJob::DELETE;
 	 break;
-      case(256+'y'):
+      case(OPT_NO_SYMLINKS):
 	 flags|=MirrorJob::NO_SYMLINKS;
 	 break;
-      case(256+'l'):
+      case(OPT_LOOP):
 	 flags|=MirrorJob::LOOP;
 	 break;
-      case(256+'E'):
+      case(OPT_MAX_ERRORS):
 	 max_error_count=atoi(optarg);
 	 break;
+      case(OPT_SKIP_NOACCESS):
+	 skip_noaccess=true;
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
       no_job:
@@ -1492,6 +1537,8 @@ CMD(mirror)
    {
       exclude=new PatternSet;
       exclude->Add(PatternSet::EXCLUDE,new PatternSet::Regex(default_exclude));
+      if(default_include && *default_include)
+	 exclude->Add(PatternSet::INCLUDE,new PatternSet::Regex(default_include));
    }
 
    args->back();
@@ -1582,6 +1629,8 @@ CMD(mirror)
    j->UseCache(use_cache);
    if(remove_source_files)
       j->RemoveSourceFiles();
+   if(skip_noaccess)
+      j->SkipNoAccess();
    if(parallel<0)
       parallel=0;
    if(parallel>64)
