@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2003 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2003-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.cc,v 1.51 2005/11/30 13:51:26 lav Exp $ */
+/* $Id: SFtp.cc,v 1.59 2006/06/13 14:35:40 lav Exp $ */
 
 #include <config.h>
 #include "SFtp.h"
@@ -76,15 +76,11 @@ int SFtp::Do()
    int s;
 
    // check if idle time exceeded
-   if(mode==CLOSED && send_buf && idle>0)
+   if(mode==CLOSED && send_buf && idle_timer.Stopped())
    {
-      if(now-idle_start>=idle)
-      {
-	 DebugPrint("---- ",_("Closing idle connection"),1);
-	 Disconnect();
-	 return m;
-      }
-      TimeoutS(idle_start+idle-time_t(now));
+      DebugPrint("---- ",_("Closing idle connection"),1);
+      Disconnect();
+      return m;
    }
 
    if(Error())
@@ -104,13 +100,13 @@ int SFtp::Do()
       return m;
 
    if(send_buf)
-      BumpEventTime(send_buf->EventTime());
+      timeout_timer.Reset(send_buf->EventTime());
    if(recv_buf)
-      BumpEventTime(recv_buf->EventTime());
+      timeout_timer.Reset(recv_buf->EventTime());
    if(pty_send_buf)
-      BumpEventTime(pty_send_buf->EventTime());
+      timeout_timer.Reset(pty_send_buf->EventTime());
    if(pty_recv_buf)
-      BumpEventTime(pty_recv_buf->EventTime());
+      timeout_timer.Reset(pty_recv_buf->EventTime());
 
    // check for timeout only if there should be connection activity.
    if(state!=DISCONNECTED && state!=CONNECTED
@@ -193,7 +189,7 @@ int SFtp::Do()
       ssh=new PtyShell(cmd);
       ssh->UsePipes();
       state=CONNECTING;
-      event_time=now;
+      timeout_timer.Reset();
       m=MOVED;
    }
    case CONNECTING:
@@ -314,7 +310,7 @@ void SFtp::MoveConnectionHere(SFtp *o)
    if(expect_chain_end==&o->expect_chain)
       expect_chain_end=&expect_chain;
    o->expect_chain_end=&o->expect_chain;
-   BumpEventTime(o->event_time);
+   timeout_timer.Reset(o->timeout_timer);
    ssh_id=o->ssh_id;
    state=CONNECTED;
    o->Disconnect();
@@ -739,7 +735,7 @@ int SFtp::HandlePty()
 	    SetError(LOGIN_FAILED,_("Password required"));
 	    return MOVED;
 	 }
-	 if(password_sent>0)
+	 if(password_sent>1)
 	 {
 	    SetError(LOGIN_FAILED,_("Login incorrect"));
 	    return MOVED;
@@ -789,12 +785,17 @@ void SFtp::HandleExpect(Expect *e)
       {
 	 protocol_version=((Reply_VERSION*)reply)->GetVersion();
 	 Log::global->Format(9,"---- protocol version set to %d\n",protocol_version);
+	 const char *charset=0;
 	 if(protocol_version>=4)
+	    charset="UTF-8";
+	 else
+	    charset=ResMgr::Query("sftp:charset",hostname);
+	 if(charset && *charset)
 	 {
 	    send_translate=new DirectedBuffer(DirectedBuffer::PUT);
 	    recv_translate=new DirectedBuffer(DirectedBuffer::GET);
-	    send_translate->SetTranslation("UTF-8",false);
-	    recv_translate->SetTranslation("UTF-8",true);
+	    send_translate->SetTranslation(charset,false);
+	    recv_translate->SetTranslation(charset,true);
 	 }
       }
       else
@@ -815,7 +816,7 @@ void SFtp::HandleExpect(Expect *e)
 	    PropagateHomeAuto();
 	    if(!home)
 	       set_home(home_auto);
-	    LsCache::SetDirectory(this, home, true);
+	    cache->SetDirectory(this, home, true);
 	 }
       }
       break;
@@ -825,7 +826,7 @@ void SFtp::HandleExpect(Expect *e)
 	 const FileAttrs *a=((Reply_ATTRS*)reply)->GetAttrs();
 	 if(a->type!=SSH_FILEXFER_TYPE_DIRECTORY)
 	 {
-	    LsCache::SetDirectory(this,cwd,false);
+	    cache->SetDirectory(this,cwd,false);
 	    SetError(NO_FILE,strerror(ENOTDIR));
 	    break;
 	 }
@@ -833,7 +834,7 @@ void SFtp::HandleExpect(Expect *e)
 	 {
 	    cwd.Set(file);
 	    eof=true;
-	    LsCache::SetDirectory(this,cwd,true);
+	    cache->SetDirectory(this,cwd,true);
 	 }
       }
       else
@@ -1336,7 +1337,7 @@ const char *SFtp::CurrentStatus()
    return "";
 }
 
-bool SFtp::SameSiteAs(FileAccess *fa)
+bool SFtp::SameSiteAs(const FileAccess *fa) const
 {
    if(!SameProtoAs(fa))
       return false;
@@ -1345,7 +1346,7 @@ bool SFtp::SameSiteAs(FileAccess *fa)
    && !xstrcmp(user,o->user) && !xstrcmp(pass,o->pass));
 }
 
-bool SFtp::SameLocationAs(FileAccess *fa)
+bool SFtp::SameLocationAs(const FileAccess *fa) const
 {
    if(!SameSiteAs(fa))
       return false;
@@ -1389,6 +1390,21 @@ void SFtp::Reconfig(const char *name)
       size_read=16;
    if(size_write<16)
       size_write=16;
+   if(!xstrcmp(name,"sftp:charset") && protocol_version && protocol_version<4)
+   {
+      if(!IsSuspended())
+	 cache->TreeChanged(this,"/");
+      const char *charset=ResMgr::Query("fish:charset",hostname);
+      if(charset && *charset)
+      {
+	 if(!send_translate)
+	    send_translate=new DirectedBuffer(DirectedBuffer::PUT);
+	 if(!recv_translate)
+	    recv_translate=new DirectedBuffer(DirectedBuffer::GET);
+	 send_translate->SetTranslation(charset,false);
+	 recv_translate->SetTranslation(charset,true);
+      }
+   }
 }
 
 void SFtp::ClassInit()
@@ -2025,8 +2041,9 @@ int SFtpDirList::Do()
       const char *cache_buffer=0;
       int cache_buffer_size=0;
       int err;
-      if(use_cache && LsCache::Find(session,dir,FA::LONG_LIST,&err,
-				    &cache_buffer,&cache_buffer_size,&fset))
+      const FileSet *fset_c;
+      if(use_cache && FileAccess::cache->Find(session,dir,FA::LONG_LIST,&err,
+				    &cache_buffer,&cache_buffer_size,&fset_c))
       {
 	 if(err)
 	 {
@@ -2036,14 +2053,14 @@ int SFtpDirList::Do()
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
-	 fset=new FileSet(fset);
+	 fset=new FileSet(fset_c);
       }
       else
       {
 	 session->Open(dir,FA::LONG_LIST);
 	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
+	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
+	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
 
@@ -2054,7 +2071,7 @@ int SFtpDirList::Do()
    {
       if(!fset && session->IsOpen())
 	 fset=((SFtp*)session)->GetFileSet();
-      LsCache::Add(session,dir,FA::LONG_LIST,FA::OK,ubuf,fset);
+      FileAccess::cache->Add(session,dir,FA::LONG_LIST,FA::OK,ubuf,fset);
       if(use_file_set)
       {
 	 fset->Sort(fset->BYNAME,false);
@@ -2168,8 +2185,9 @@ int SFtpListInfo::Do()
       const char *cache_buffer=0;
       int cache_buffer_size=0;
       int err;
-      if(use_cache && LsCache::Find(session,"",FA::LONG_LIST,&err,
-				    &cache_buffer,&cache_buffer_size,&result))
+      const FileSet *fset_c;
+      if(use_cache && FileAccess::cache->Find(session,"",FA::LONG_LIST,&err,
+				    &cache_buffer,&cache_buffer_size,&fset_c))
       {
 	 if(err)
 	 {
@@ -2179,14 +2197,14 @@ int SFtpListInfo::Do()
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
-	 result=new FileSet(result);
+	 result=new FileSet(fset_c);
       }
       else
       {
 	 session->Open("",FA::LONG_LIST);
 	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
+	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
+	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
    const char *b;
@@ -2196,7 +2214,7 @@ int SFtpListInfo::Do()
    {
       if(!result && session->IsOpen())
 	 result=((SFtp*)session)->GetFileSet();
-      LsCache::Add(session,"",FA::LONG_LIST,FA::OK,ubuf,result);
+      FileAccess::cache->Add(session,"",FA::LONG_LIST,FA::OK,ubuf,result);
       result->ExcludeDots();
       result->Exclude(exclude_prefix,exclude);
       done=true;
