@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.h,v 1.5 2001/05/29 06:23:00 lav Exp $ */
+/* $Id: SleepJob.h,v 1.11 2006/06/16 07:30:53 lav Exp $ */
 
 #ifndef SLEEPJOB_H
 #define SLEEPJOB_H
@@ -26,29 +26,34 @@
 #include "Job.h"
 #include "ResMgr.h"
 
-class SleepJob : public SessionJob
+class LocalDirectory;
+
+class SleepJob : public SessionJob, public Timer
 {
-   time_t start_time;
-   TimeInterval next_time;
    char *cmd;
    int exit_code;
    bool done;
-   char *saved_cwd;
+   LocalDirectory *saved_cwd;
    class CmdExec *exec;
-   bool   repeat;
-   int    repeat_count;
+   bool repeat;
+   int repeat_count;
+   int max_repeat_count;
+   int continue_code;
+   int break_code;
 
 public:
    int Do();
    int Done() { return done; }
    int ExitCode() { return exit_code; }
 
-   SleepJob(const TimeInterval &when,FileAccess *s=0,char *what=0);
+   SleepJob(const TimeInterval &when,FileAccess *s=0,LocalDirectory *cwd=0,char *what=0);
    ~SleepJob();
 
-   void PrintStatus(int v);
+   void PrintStatus(int v,const char *);
 
-   void Repeat() { repeat=true; start_time-=next_time.Seconds(); }
+   void Repeat(int m) { repeat=true; max_repeat_count=m; Stop(); }
+   void ContinueCode(int c) { continue_code=c; }
+   void BreakCode(int c) { break_code=c; }
 
    void lftpMovesToBackground();
 };
