@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -206,7 +206,7 @@ Buffer::~Buffer()
 {
    xfree(error_text);
    xfree(buffer);
-   delete rate;
+   SMTask::Delete(rate);
 }
 
 const char *Buffer::GetRateStrS()
@@ -385,6 +385,8 @@ int IOBuffer::Do()
       break;
 
    case GET:
+      if(eof)
+	 return STALL;
       res=Get_LL(GET_BUFSIZE);
       if(res>0)
       {
@@ -443,6 +445,8 @@ int IOBufferStacked::Do()
 
    case GET:
       m|=down->Do();
+      if(eof)
+	 return m;
       res=Get_LL(GET_BUFSIZE);
       if(res>0)
       {
@@ -636,15 +640,13 @@ int IOBufferFileAccess::Get_LL(int size)
    return res;
 }
 
-void IOBufferFileAccess::Suspend()
+void IOBufferFileAccess::SuspendInternal()
 {
-   session->Suspend();
-   super::Suspend();
+   session->SuspendSlave();
 }
-void IOBufferFileAccess::Resume()
+void IOBufferFileAccess::ResumeInternal()
 {
-   super::Resume();
-   session->Resume();
+   session->ResumeSlave();
 }
 const char *IOBufferFileAccess::Status()
 {
